library geiger_localstorage;

import 'package:geiger_localstorage/geiger_localstorage.dart';

/// <p>Field reflects the available ordinals in a node.</p>
class Field {
  static const Field owner = Field._(1, 'owner', ComparatorType.string);
  static const Field name = Field._(2, 'name', ComparatorType.string);
  static const Field path = Field._(3, 'path', ComparatorType.string);
  static const Field key = Field._(4, 'key', ComparatorType.string);
  static const Field value = Field._(5, 'value', ComparatorType.string);
  static const Field type = Field._(6, 'type', ComparatorType.string);
  static const Field visibility =
      Field._(7, 'visibility', ComparatorType.string);
  static const Field lastModified =
      Field._(8, 'lastModified', ComparatorType.datetime);
  static const Field expiry = Field._(9, 'expiry', ComparatorType.datetime);
  static const Field tombstone =
      Field._(10, 'tombstone', ComparatorType.boolean);

  static const Map<String, Field> _values = {
    'OWNER': owner,
    'NAME': name,
    'PATH': path,
    'KEY': key,
    'VALUE': value,
    'TYPE': type,
    'VISIBILITY': visibility,
    'LASTMODIFIED': lastModified,
    'EXPIRY': expiry,
    'TOMBSTONE': tombstone,
  };

  final ComparatorType _comparator;
  final int _id;
  final String _name;

  ComparatorType get comparator => _comparator;

  int get id => _id;

  const Field._(this._id, this._name, this._comparator);

  static List<Field> get values {
    return _values.values.toList();
  }

  @override
  String toString() {
    return _name;
  }

  static Field? fromString(String value) {
    var ret = Field._values[value.toUpperCase()];
    return ret;
  }
}
