// ignore_for_file: avoid_print

import 'package:dependency_visitor/dependency_visitor.dart';
import 'package:flutter/widgets.dart';
import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:test/test.dart';

Future<String?> getFile() async {
  String? ret;
  await DependencyVisitor(filePaths: ['assets/store/store.data'])
      .run()
      .forEach((dependencyFile) {
    ret = dependencyFile.content;
  });
  return ret;
}

void main() async {
  group('node_parser tests', () {
    test('availability of asset (1)', () async {
      WidgetsFlutterBinding.ensureInitialized();
      String? initialNodes = await GenericController.getResourceOrFile(null);
      expect(initialNodes, isNotEmpty,
          reason: 'initial store asset not loaded');
    });
    test('availability of asset (2)', () async {
      String initialNodes =
          await GenericController.getResourceOrFile('store.data');
      expect(initialNodes, isNotEmpty,
          reason: 'initial store asset not loaded');
    });
    test('parse default asset', () async {
      String initialNodes =
          await GenericController.getResourceOrFile('assets/store/store.data');
      List<Node> ln = NodeParser(initialNodes).getNodes();
      expect(ln.length > 45, isTrue,
          reason: 'Number of nodes is too low (got only ${ln.length} nodes)');
    });
    test('parse dump database', () async {
      String initialNodes =
          await GenericController('test', DummyMapper('test')).dump();
      List<Node> ln = NodeParser(initialNodes).getNodes();
      for (Node n in ln) {
        print('## $n');
      }
      expect(ln.length > 50, isTrue,
          reason: 'Number of nodes is too low (got only ${ln.length} nodes)');
    });
  });
}
