// ignore_for_file: prefer_const_constructors

import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:test/test.dart';

class NodeTest {}

void main() async {
  group('node base functionality tests', () {
    test('Test illegal node path', () {
      expect(
          () => NodeImpl('', 'owner'), throwsA(TypeMatcher<StorageException>()),
          reason: 'illegal null name without path');
      expect(() => NodeImpl('', 'owner', ':path'),
          throwsA(TypeMatcher<StorageException>()),
          reason: 'path specified without name');
      expect(() => NodeImpl('a:b', 'owner', ':path'),
          throwsA(TypeMatcher<StorageException>()),
          reason: 'path specified with a name containing delimiters');
    });
    test('node root parent path case', () async {
      Node node = NodeImpl(':', 'owner');
      expect(node.parentPath, null,
          reason: 'parent path of root node is not null');
    });

    test('node default visibility', () async {
      Node node = NodeImpl('name', 'owner', ':path');
      expect(node.visibility, Visibility.red, reason: 'test predefined owner');
    });

    test('node equality equals() after deep clone', () async {
      Node node = NodeImpl('name', 'owner', ':path');
      Node node2 = await node.deepClone();
      expect(node.equals(node2), true,
          reason: 'equality not given original:$node; clone:$node2');
    });

    test('ordinals mismatch', () async {
      Node node = NodeImpl('name', 'owner', ':path');
      Node node2 = await node.deepClone();
      node2.owner = 'newOwner';
      expect(node.equals(node2), false);
    });

    test('equality for child nodes', () async {
      Node node = NodeImpl('name', 'owner', ':path');
      Node node2 = await node.deepClone();
      await node2.addChild(NodeImpl('name2', 'owner', ':path:name'));
      expect(node.equals(node2), false);
    });

    test('node serialization', () async {
      Node node = NodeImpl('name', 'owner', ':path');
      ByteSink sink = ByteSink();
      node.toByteArrayStream(sink);
      sink.close();
      ByteStream stream = ByteStream(null, await sink.bytes);
      Node n2 = await NodeImpl.fromByteArrayStream(stream);
      expect(n2.equals(node), isTrue);
    });

    group('visibility testing', () {
      test('node defaults', () async {
        Node node = NodeImpl('name', 'owner', ':path', Visibility.green);
        expect(node.visibility, Visibility.green,
            reason: 'test visibility persistence failed');
      });
    });
  });
}
