// ignore_for_file: avoid_print

import 'dart:async';

import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:test/test.dart';

void _callIncliningTests(bool requireInit) {
  test('test strict inclining timestamp with $requireInit', () {
    int lastTimestamp = 0;
    for (int i = 0; i < 1000; i++) {
      int ts = ExtendedTimestamp.now(requireInit);
      expect(ts > lastTimestamp, isTrue, reason: 'not strictly inclining');
    }
  });
  test('test follow of ExtendedTimestamp', () async {
    String lastts = '';
    StorageController controller =
        GenericController('test', DummyMapper('test'));
    await ExtendedTimestamp.initializeTimestamp(controller);
    for (int i = 0; i < 1000; i++) {
      String ts = ExtendedTimestamp.convertToExtendedTimestamp(
          ExtendedTimestamp.now(requireInit));
      expect(ts.compareTo(lastts), 1, reason: 'not strictly inclining');
    }
  });
}

Future<void> main() async {
  group('un-inited timestamp tests', () {
    _callIncliningTests(false);
  });
  group('inited timestamp tests', () {
    StorageController c =
        GenericController('testowner', DummyMapper('testOwner'));
    test('testing un-inited timestamp exception', () async {
      await c.zap();
      ExtendedTimestamp.reset();
      expect(() => ExtendedTimestamp.now(),
          throwsA(const TypeMatcher<StorageException>()),
          reason: 'unexpectedly successful on un-inited ExtendedTimestamp');
    });
    test('Calling init', () async {
      await ExtendedTimestamp.initializeTimestamp(c);
    });
    _callIncliningTests(true);
  });
  group('conversion tests', () {
    test('Circular conversion', () async {
      // create a fresh database
      StorageController c =
          GenericController('testowner', DummyMapper('testOwner'));
      ExtendedTimestamp.reset();
      await c.zap();
      await ExtendedTimestamp.initializeTimestamp(c);

      // get a timestamp
      String sts = ExtendedTimestamp.extendedNow(true);

      // do it once forward and backward again
      expect(
          sts,
          ExtendedTimestamp.convertToExtendedTimestamp(
              ExtendedTimestamp.convertFromExtendedTimestamp(sts)));

      // test with broken timestamps
      for (String s in [
        sts.split('-')[0],
      ]) {
        expect(() => ExtendedTimestamp.convertFromExtendedTimestamp(s),
            throwsA(const TypeMatcher<StorageException>()),
            reason: 'unexpectedly successful on illegal timestamp "$s"');
      }
    });
  });
}
