// ignore_for_file: avoid_print

import 'dart:async';

import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:intl/locale.dart';
import 'package:logging/logging.dart';
import 'package:test/test.dart';

class UserTest {}

Future<void> luongTests() async {
  group('luongnv89 tests', () {
    test('20210929 - test1', () async {
      await StorageMapper.initDatabaseExpander();
      StorageController geigerToolboxStorageController =
          GenericController('MI-Cyberrange', SqliteMapper('./testdb.sqlite'));
      await geigerToolboxStorageController.close();
      // File('dbFileName.sqlite').deleteSync();
    });
    test('20210929 - test2', () async {
      // init
      await StorageMapper.initDatabaseExpander();
      StorageController geigerToolboxStorageController =
          GenericController('MI-Cyberrange', DummyMapper('testowner'));

      // sample code
      var newScore = 123;
      var currentUserLevel = 123;
      try {
        Node node = await geigerToolboxStorageController.get(':score-node');
        // this is bad coding as it depends on a full transaction database state. Introduced addOrUpdateValue as drop in replacement.
        node.addValue(NodeValueImpl('score', '$newScore'));
        node.addValue(NodeValueImpl('level', '$currentUserLevel'));
        await geigerToolboxStorageController.update(node);
      } catch (e) {
        print('Exception while creating node');
        Node node = NodeImpl(':score-node', 'testowner');
        await geigerToolboxStorageController.add(node);
        node.addValue(NodeValueImpl('score', '$newScore'));
        node.addValue(NodeValueImpl('level', '$currentUserLevel'));
        await geigerToolboxStorageController.update(node);
      }
    });
  });
}

class Event {
  final EventType _event;
  final Node? _old;
  final Node? _new;

  Event(this._event, this._old, this._new);

  EventType get type => _event;

  Node? get oldNode => _old;

  Node? get newNode => _new;

  @override
  String toString() {
    return '${type.toString()} ${oldNode.toString()}=>${newNode.toString()}';
  }
}

class CustomStorageListener implements StorageListener {
  List<Event> events = <Event>[];

  @override
  void gotStorageChange(EventType event, Node? oldNode, Node? newNode) {
    Event e = Event(event, oldNode, newNode);
    events.add(e);
    print('got event ${e.toString()}');
  }

  Future<List<Event>> getNumEvents(int num, [int timeout = 1000000]) async {
    int start = DateTime.now().millisecondsSinceEpoch;
    List<Event> ret = await Future.doWhile(() =>
            events.length < num &&
            start + 1000 * timeout > DateTime.now().millisecondsSinceEpoch)
        .then((value) => events);
    if (events.length < num) {
      throw TimeoutException('Timeout reached while waiting for $num events');
    }
    return ret;
  }
}

Future<void> nosaTests() async {
  final Logger _logger = Logger('GeigerStorage');
  group('nosadaniel tests', () {
    test(
        '20211012 - flutter Github Issue 16 (https://github.com/cyber-geiger/toolbox-storage/issues/16)',
        () async {
      StorageController _storageController =
          GenericController("Example", SqliteMapper("./testdb.sqlite"));
      await _storageController.zap();
      CustomStorageListener l = CustomStorageListener();
      SearchCriteria s = SearchCriteria(searchPath: ':');
      await _storageController.registerChangeListener(l, s);
      Node n = NodeImpl(':hello', 'testowner');
      expect(await s.evaluate(n), isTrue,
          reason: 'node fails to evaluate successfully');
      List<Event> e = await _storageController
          .add(n)
          .then((_) async => await l.getNumEvents(1, 2));
      expect(e.length, 1, reason: 'Did not get expected number of results');
    });
    test('20211119 - flutter Github issue #18', () async {
      //initialize database
      await StorageMapper.initDatabaseExpander();
      StorageController _storageController =
          GenericController("Example", SqliteMapper("./testdb.sqlite"));
      await _storageController.zap();

      Future<void> setGlobalThreatsNode() async {
        List<String> threats = ["phishing", "malware"];
        List<String> threatid = ["t1", "t2"];
        try {
          for (int i = 0; i < threats.length; i++) {
            Node _node =
                await _storageController.get(':Global:threats:${threatid[i]}');
            //create a NodeValue
            NodeValue threatNodeValueName = NodeValueImpl("name", threats[i]);
            await _node.addOrUpdateValue(threatNodeValueName);
            await _storageController.update(_node);
          }
        } on StorageException {
          //print(":Global:threats not found");
          try {
            Node threatsNode = NodeImpl(":Global:threats", "testowner");
            await _storageController.addOrUpdate(threatsNode);

            int numThreats = threats.length;
            for (int i = 0; i < numThreats; i++) {
              Node threatIdNode =
                  NodeImpl(":Global:threats:${threatid[i]}", "testowner");
              //create :Global:threats:$threatId
              await _storageController.addOrUpdate(threatIdNode);
              //create a NodeValue
              NodeValue threatNodeValueName = NodeValueImpl("name", threats[i]);
              await threatIdNode.addOrUpdateValue(threatNodeValueName);
              await _storageController.update(threatIdNode);
              print('## added $threatIdNode');
            }
          } catch (e, st) {
            print(
                "*********************************************************************");
            print(e);
            print(st);
            fail('failed due to unexpected exception');
          }
        }
      }

      Future<List<String>> getThreats({String language = "en"}) async {
        List<String> t = [];
        try {
          Node _node = await _storageController.get(":Global:threats");

          //return _node!.getChildNodesCsv();
          String children = await _node.getChildNodesCsv();
          print('## csv is $children');
          List<String> i = children.split(',');
          expect(i.length, 2, reason: 'List of children has unexpected size');
          for (String threatId in i) {
            Node threatNode =
                await _storageController.get(":Global:threats:$threatId");

            t.add(await threatNode
                .getValue("name")
                .then((value) => value!.getValue(language)!));
          }
          print('## got ${t.length} threats from the storage');
        } on StorageException catch (e, st) {
          _logger.log(Level.INFO,
              'Got Storage Exception while fetching list of threats', e, st);
          print(e);
          print(st);
          fail('Got Storage Exception while fetching list of threats');
        } on Error catch (e) {
          print(
              "*********************************************************************");
          print('$e');
          fail('Got Exception while fetching list of threats');
        }
        return t;
      }

      Future<List<String>> getListThreat() async {
        await setGlobalThreatsNode();
        return await getThreats();
      }

      print("Global Threats: ${await getListThreat()}");
    });
    test('20211119 - flutter Github issue #18 (rewrite)', () async {
      //initialize database
      await StorageMapper.initDatabaseExpander();
      StorageController _storageController =
          GenericController("Example", SqliteMapper("./testdb.sqlite"));
      // Make sure we start off with a fresh DB
      await _storageController.zap();

      Future<void> setGlobalThreatsNode() async {
        Map<String, String> threats = {"t1": "phishing", "t2": "malware"};
        try {
          // Make sure that parent node exists
          Node threatsNode = NodeImpl(":Global:threats", "testowner");
          await _storageController.addOrUpdate(threatsNode);

          // create threats
          for (MapEntry<String, String> e in threats.entries) {
            // create node
            Node _node = NodeImpl(':Global:threats:${e.key}', 'testowner');
            await _node.addOrUpdateValue(NodeValueImpl("name", e.value));
            await _storageController.addOrUpdate(_node);
          }
        } on StorageException catch (e, st) {
          print(
              "*********************************************************************");
          print(e);
          print(st);
          fail('failed due to unexpected exception');
        }
      }

      Future<List<String>> getThreats({String language = "en"}) async {
        List<String> t = [];
        try {
          List<Node> _node = await _storageController
              .search(SearchCriteria(searchPath: ":Global:threats"));
          for (Node element in _node) {
            if (element.parentPath == ":Global:threats") {
              t.add((await element.getValue('name'))!.value);
            }
          }
        } on Error catch (e) {
          print(
              "*********************************************************************");
          print('$e');
          fail('Got Exception while fetching list of threats');
        }
        return t;
      }

      Future<List<String>> getListThreat() async {
        await setGlobalThreatsNode();
        return await getThreats();
      }

      print("Global Threats: ${await getListThreat()}");
    });
    test(
        '20211129 - Unhandled Exception: Null check operator used on a null value Error #24',
        () async {
      Future<StorageController?> initGeigerStorage() async {
        try {
          await StorageMapper.initDatabaseExpander();
          StorageController geigerToolboxStorageController =
              GenericController('owner', SqliteMapper('./testdb.sqlite'));
          return geigerToolboxStorageController;
        } catch (e) {
          print("Failed to connect to database");
          fail('unable to open database');
        }
      }

      Future<void> writeToGeigerStorage(
          StorageController geigerToolboxStorageController, String data) async {
        print('Trying to get the data node');
        String dataNodeName = ':dataNode';
        try {
          print('Found the data node - Going to write the data');
          Node node = await geigerToolboxStorageController.get(dataNodeName);
          await node.updateValue(NodeValueImpl('data', data));
          await geigerToolboxStorageController.update(node);
        } catch (e) {
          print(e.toString());
          print('Cannot find the data node - Going to create a new one');
          try {
            Node node = NodeImpl(dataNodeName, 'owner');
            node.visibility = Visibility.green;

            await geigerToolboxStorageController.addOrUpdate(node);
            await node.addOrUpdateValue(NodeValueImpl('data', data));
            await geigerToolboxStorageController.update(node);
          } catch (e2, s2) {
            print(e2.toString());
            print(s2.toString());
            fail('out of luck');
          }
        }
      }

      Future<String?> readDataFromGeigerStorage(
          StorageController geigerToolboxStorageController) async {
        String nodeDataName = ':dataNode';
        print('Trying to get the data node');
        try {
          print('Found the data node - Going to get the data');
          Node node = await geigerToolboxStorageController.get(nodeDataName);
          List<Node> nodes = await geigerToolboxStorageController
              .search(SearchCriteria(searchPath: nodeDataName));
          print("list of path node: $nodes");
          NodeValue? nValue = await node.getValue('data');
          if (nValue != null) {
            return nValue.value;
          } else {
            fail('no data found in node value');
          }
        } catch (e, s) {
          print(e.toString());
          print(s.toString());
          fail('Failed to retrieve the data node');
        }
      }

      try {
        StorageController? c = await initGeigerStorage();
        await writeToGeigerStorage(c!, "nosa");
        String? result = await readDataFromGeigerStorage(c);
        expect(result, 'nosa');
      } catch (e) {
        fail('failed due to unexpected exception');
      }
    });
    test(
        '20211129 - Unhandled Exception: Null check operator used on a null value Error #24 (rewrite)',
        () async {
      try {
        await StorageMapper.initDatabaseExpander();
        StorageController c =
            GenericController('owner', SqliteMapper('./testdb.sqlite'));
        await ExtendedTimestamp.initializeTimestamp(c);
        await c.addOrUpdateValue(':dataNode', NodeValueImpl('data', 'nosa'));
        String? result =
            (await c.getValue(':dataNode', 'data'))!.toSimpleString();
        expect(result, 'nosa');
      } catch (e) {
        fail('failed due to unexpected exception');
      }
    });
    test('20211207 - Sample code listening for update', () async {
      // setup test scenario
      StorageController _storageController =
          GenericController("Example", SqliteMapper('./testdb.sqlite'));
      await _storageController.zap();
      CustomStorageListener l = CustomStorageListener();
      SearchCriteria s = SearchCriteria(searchPath: ':');
      await _storageController.registerChangeListener(l, s);
      Node n = NodeImpl(':hello', 'testowner');
      expect(await s.evaluate(n), isTrue,
          reason: 'node fails to evaluate successfully');
      await _storageController.add(n);
      Node n2 = await n.deepClone();
      await n2.addValue(NodeValueImpl('key', 'value'));

      // do the update (and wait for the event to happen; max 2s)
      List<Event> e = await _storageController
          .update(n2)
          .then((_) async => await l.getNumEvents(2, 2000));

      // check if two event received
      expect(e.length, 2, reason: 'Did not get expected number of results');

      // check add event information
      expect(e[0].type, EventType.create,
          reason: 'Did not get expected type (create)');
      expect(e[0].oldNode, isNull, reason: 'Did not get empty predecessor');
      expect(e[0].newNode, n, reason: 'Did not get expected node content');

      // check update event information
      expect(e[1].type, EventType.update,
          reason: 'Did not get expected type (update)');
      expect(e[1].oldNode, n, reason: 'Did not get empty predecessor');
      expect(e[1].newNode, n2, reason: 'Did not get expected node content');
    });
  });
}

class TestListener1 implements StorageListener {
  // Update on currentDeviceChange the networkInterface uuid
  @override
  void gotStorageChange(EventType event, Node? oldNode, Node? newNode) async {
    String? value = (await newNode!.getValue("currentDevice"))!.value;
    print(value);
  }
}

Future<void> nicolaeschbachTests() async {
  group('nicolaeschbach tests', () {
    test('20220118 - Register NodeValue Change Listener', () async {
      StorageController controller =
          GenericController("testOwner", DummyMapper("testOwner"));
      SearchCriteria criteria =
          SearchCriteria(searchPath: ":Local", key: "currentDevice");
      controller.registerChangeListener(TestListener1(), criteria);
      controller.updateValue(
          ":Local", NodeValueImpl("currentDevice", "newDeviceUUID"));
    });
    test('get NodeValue doesnt return the correct nodeValue lastModified ',
        () async {
      Node node = NodeImpl(':TestNode', 'testOwner');
      NodeValue value = NodeValueImpl('Key1', 'Value');
      await node.addValue(value);
      // delayed just to show the time difference
      await Future.delayed(const Duration(milliseconds: 500));
      NodeValue valueCopy = (await node.getValue("Key1"))!;
      expect(value.lastModified, valueCopy.lastModified);
    });
  });
}

Future<void> reuvenTests() async {
  group('BananaEgg tests', () {
    test(
        '20211209 - Github issue cyber-geiger/toolbox-communicationApi#3 (cannot get node values)',
        () async {
      String nodeName = ':Enterprise:myTest';

      // setup an empty database with a node containing multiple values
      await StorageMapper.initDatabaseExpander();
      StorageController controller =
          GenericController("testOwner", SqliteMapper('./testdb.sqlite'));
      Node node1 = NodeImpl(nodeName, "testOwner");
      node1.addValue(NodeValueImpl('key1', 'value1'));
      node1.addValue(NodeValueImpl('key2', 'value2'));
      node1.addValue(NodeValueImpl('key3', 'value3'));
      NodeValue nv = NodeValueImpl('key4', 'value4');
      nv.setValue('Wert4', Locale.parse('de-de'));
      nv.setValue('Waarde4', Locale.parse('nl'));
      nv.setValue('valeur4', Locale.parse('fr'));
      node1.addValue(nv);
      await controller.zap();

      // this is just writing and fetching but we could have done it without it
      await controller.add(node1);
      node1 = await controller.get(nodeName);

      // now for the sample codes

      // fetch a specific value in the default locale
      expect((await node1.getValue('key4'))!.value, 'value4');
      // fetch a specific value in a specific locale (or its closest match)
      expect((await node1.getValue('key4'))!.getValue('de-de'), 'Wert4');

      // fetch all values and all translations by enumerating a map
      for (NodeValue v in (await node1.getValues()).values) {
        print('${v.key} = {');
        for (MapEntry<Locale, String> lang in nv.allValueTranslations.entries) {
          print('  ${lang.key.toLanguageTag()}:${lang.value}');
        }
        print('}');
      }

      await controller.close();
    });
    int size = 10 * 1024 * 1024 - 200;
    test('huge string serialization', () async {
      print('## creating huge');
      String s = "abcdefghijklmnopqrstuvwxyz";
      for (; s.length < size;) {
        s = s + s;
      }
      s = s.substring(0, size);
      int start = DateTime.now().millisecondsSinceEpoch;
      int i = DateTime.now().millisecondsSinceEpoch;
      print('## serializing string');
      ByteSink bout = ByteSink();
      SerializerHelper.writeString(bout, s);
      print(
          '## serialization done in ${DateTime.now().millisecondsSinceEpoch - i} ms');
      i = DateTime.now().millisecondsSinceEpoch;
      bout.close();
      ByteStream bin = ByteStream(null, await bout.bytes);
      String? s2 = await SerializerHelper.readString(bin);
      print(
          '## deserialization done in ${DateTime.now().millisecondsSinceEpoch - i} ms');
      int time = DateTime.now().millisecondsSinceEpoch - start;
      expect(s, s2);
      print("## done in $time ms (Total)");
    });
  });
}

Future<void> cftnTests() async {
  group("cftn tests", () {
    test(":Global:threats exists", () async {
      const owner = 'owner';
      await StorageMapper.initDatabaseExpander();
      final controller = GenericController(owner, DummyMapper(owner));
      expect(await controller.get(":Global:threats"), isNotNull);
    });
  });
}

Future<void> bananaEggTests() async {
  group("bananaEgg tests", () {
    test("SqliteMapper - node.getValues() when node from parent.getChildren()",
        () async {
      const owner = 'owner';
      await StorageMapper.initDatabaseExpander();
      final controller = GenericController(owner, SqliteMapper('database.db'));
      await controller.zap();

      final node = NodeImpl('testNode', owner, ':');
      final value = NodeValueImpl('testValue', 'test');
      await node.addValue(value);

      await controller.add(node);
      final parent = await controller.get(':');
      final children = await parent.getChildren();
      final values = await children[node.name]!.getValues();
      expect(values[value.key], value);
    });
  });
}

Future<void> main() async {
  await nicolaeschbachTests();
  await luongTests();
  await nosaTests();
  await reuvenTests();
  await cftnTests();
  await bananaEggTests();
}
